@@ -1,31 +1,38 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2007 Michal Trojnara <Michal.Trojnara@mirt.net>
- *                 All Rights Reserved
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   Copyright (C) 1998-2008 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
+ *   This program is free software; you can redistribute it and/or modify it
+ *   under the terms of the GNU General Public License as published by the
+ *   Free Software Foundation; either version 2 of the License, or (at your
+ *   option) any later version.
+ * 
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- *   In addition, as a special exception, Michal Trojnara gives
- *   permission to link the code of this program with the OpenSSL
- *   library (or with modified versions of OpenSSL that use the same
- *   license as OpenSSL), and distribute linked combinations including
- *   the two.  You must obey the GNU General Public License in all
- *   respects for all of the code used other than OpenSSL.  If you modify
- *   this file, you may extend this exception to your version of the
- *   file, but you are not obligated to do so.  If you do not wish to
- *   do so, delete this exception statement from your version.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *   See the GNU General Public License for more details.
+ * 
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, see <http://www.gnu.org/licenses>.
+ * 
+ *   Linking stunnel statically or dynamically with other modules is making
+ *   a combined work based on stunnel. Thus, the terms and conditions of
+ *   the GNU General Public License cover the whole combination.
+ * 
+ *   In addition, as a special exception, the copyright holder of stunnel
+ *   gives you permission to combine stunnel with free software programs or
+ *   libraries that are released under the GNU LGPL and with code included
+ *   in the standard release of OpenSSL under the OpenSSL License (or
+ *   modified versions of such code, with unchanged license). You may copy
+ *   and distribute such a system following the terms of the GNU GPL for
+ *   stunnel and the licenses of the other code concerned.
+ * 
+ *   Note that people who make modified versions of stunnel are not obligated
+ *   to grant this special exception for their modified versions; it is their
+ *   choice whether to do so. The GNU General Public License gives permission
+ *   to release a modified version without this exception; this exception
+ *   also makes it possible to release a modified version which carries
+ *   forward this exception.
  */
 
 #include "common.h"
@@ -79,7 +86,7 @@ void context_init(LOCAL_OPTIONS *section) { /* init SSL context */
     if(section->option.cert) {
         if(stat(section->key, &st)) {
             ioerror(section->key);
-            exit(1);
+            die(1);
         }
 #if !defined(USE_WIN32) && !defined(USE_OS2)
         if(st.st_mode & 7)
@@ -108,7 +115,7 @@ void context_init(LOCAL_OPTIONS *section) { /* init SSL context */
     if(section->cipher_list) {
         if (!SSL_CTX_set_cipher_list(section->ctx, section->cipher_list)) {
             sslerror("SSL_CTX_set_cipher_list");
-            exit(1);
+            die(1);
         }
     }
 #if SSLEAY_VERSION_NUMBER >= 0x00906000L
@@ -256,7 +263,7 @@ static void load_certificate(LOCAL_OPTIONS *section) {
     if(!SSL_CTX_use_certificate_chain_file(section->ctx, section->cert)) {
         s_log(LOG_ERR, "Error reading certificate file: %s", section->cert);
         sslerror("SSL_CTX_use_certificate_chain_file");
-        exit(1);
+        die(1);
     }
     s_log(LOG_DEBUG, "Certificate loaded");
 
@@ -283,12 +290,12 @@ static void load_certificate(LOCAL_OPTIONS *section) {
                     continue;
                 }
                 sslerror("ENGINE_load_private_key");
-                exit(1);
+                die(1);
             }
             if(SSL_CTX_use_PrivateKey(section->ctx, pkey))
                 break; /* success */
             sslerror("SSL_CTX_use_PrivateKey");
-            exit(1);
+            die(1);
         }
     else
 #endif
@@ -316,11 +323,11 @@ static void load_certificate(LOCAL_OPTIONS *section) {
 #else /* NO_RSA */
             sslerror("SSL_CTX_use_RSAPrivateKey_file");
 #endif /* NO_RSA */
-            exit(1);
+            die(1);
         }
     if(!SSL_CTX_check_private_key(section->ctx)) {
         sslerror("Private key does not match the certificate");
-        exit(1);
+        die(1);
     }
     s_log(LOG_DEBUG, "Private key loaded");
 }
