@@ -1,34 +1,40 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2007 Michal Trojnara <Michal.Trojnara@mirt.net>
- *                 All Rights Reserved
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   Copyright (C) 1998-2008 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
+ *   This program is free software; you can redistribute it and/or modify it
+ *   under the terms of the GNU General Public License as published by the
+ *   Free Software Foundation; either version 2 of the License, or (at your
+ *   option) any later version.
+ * 
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- *   In addition, as a special exception, Michal Trojnara gives
- *   permission to link the code of this program with the OpenSSL
- *   library (or with modified versions of OpenSSL that use the same
- *   license as OpenSSL), and distribute linked combinations including
- *   the two.  You must obey the GNU General Public License in all
- *   respects for all of the code used other than OpenSSL.  If you modify
- *   this file, you may extend this exception to your version of the
- *   file, but you are not obligated to do so.  If you do not wish to
- *   do so, delete this exception statement from your version.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *   See the GNU General Public License for more details.
+ * 
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, see <http://www.gnu.org/licenses>.
+ * 
+ *   Linking stunnel statically or dynamically with other modules is making
+ *   a combined work based on stunnel. Thus, the terms and conditions of
+ *   the GNU General Public License cover the whole combination.
+ * 
+ *   In addition, as a special exception, the copyright holder of stunnel
+ *   gives you permission to combine stunnel with free software programs or
+ *   libraries that are released under the GNU LGPL and with code included
+ *   in the standard release of OpenSSL under the OpenSSL License (or
+ *   modified versions of such code, with unchanged license). You may copy
+ *   and distribute such a system following the terms of the GNU GPL for
+ *   stunnel and the licenses of the other code concerned.
+ * 
+ *   Note that people who make modified versions of stunnel are not obligated
+ *   to grant this special exception for their modified versions; it is their
+ *   choice whether to do so. The GNU General Public License gives permission
+ *   to release a modified version without this exception; this exception
+ *   also makes it possible to release a modified version which carries
+ *   forward this exception.
  */
 
-
 #include "common.h"
 #include "prototypes.h"
 
@@ -44,7 +50,7 @@ int allow_severity=LOG_NOTICE, deny_severity=LOG_WARNING;
 static ssize_t read_fd(int, void *, size_t, int *);
 static ssize_t write_fd(int, void *, size_t, int);
 
-int nproc=0;
+int num_processes=0;
 static int *ipc_socket, *busy;
 #endif /* USE_PTHREAD */
 
@@ -55,24 +61,24 @@ void libwrap_init(int num) {
     int i, rfd, result;
     char servname[STRLEN];
 
-    nproc=num;
-    if(!nproc) /* no extra processes to spawn */
+    num_processes=num;
+    if(!num_processes) /* no extra processes to spawn */
         return;
-    ipc_socket=calloc(2*nproc, sizeof(int));
-    busy=calloc(nproc, sizeof(int));
+    ipc_socket=calloc(2*num_processes, sizeof(int));
+    busy=calloc(num_processes, sizeof(int));
     if(!ipc_socket || !busy) {
         s_log(LOG_ERR, "Memory allocation failed");
-        exit(1);
+        die(1);
     }
-    for(i=0; i<nproc; ++i) { /* spawn a child */
+    for(i=0; i<num_processes; ++i) { /* spawn a child */
         if(socketpair(AF_UNIX, SOCK_STREAM, 0, ipc_socket+2*i)) {
             sockerror("socketpair");
-            exit(1);
+            die(1);
         }
         switch(fork()) {
         case -1:    /* error */
             ioerror("fork");
-            exit(1);
+            die(1);
         case  0:    /* child */
             close(ipc_socket[2*i]); /* server side */
             while(1) { /* main libwrap client loop */
@@ -95,35 +101,72 @@ void libwrap_init(int num) {
 void auth_libwrap(CLI *c) {
     int result=0; /* deny by default */
 #ifdef USE_PTHREAD
-    static int num_busy=0, rr=0;
+    volatile static int num_busy=0, roundrobin=0;
+    int retval, my_process;
     static pthread_mutex_t mutex=PTHREAD_MUTEX_INITIALIZER;
     static pthread_cond_t cond=PTHREAD_COND_INITIALIZER;
 
-    if(nproc) {
+
+    if(num_processes) {
         s_log(LOG_DEBUG, "Waiting for a libwrap process");
 
-        pthread_mutex_lock(&mutex);
-        while(num_busy>=nproc) /* all child processes are busy */
-            pthread_cond_wait(&cond, &mutex);
-        while(busy[rr]) /* find a free child process */
-            rr=(rr+1)%nproc;
+        retval=pthread_mutex_lock(&mutex);
+        if(retval) {
+            errno=retval;
+            ioerror("pthread_mutex_lock");
+            longjmp(c->err, 1);
+        }
+        while(num_busy==num_processes) { /* all child processes are busy */
+            retval=pthread_cond_wait(&cond, &mutex);
+            if(retval) {
+                errno=retval;
+                ioerror("pthread_cond_wait");
+                longjmp(c->err, 1);
+            }
+        }
+        while(busy[roundrobin]) /* find a free child process */
+            roundrobin=(roundrobin+1)%num_processes;
+        my_process=roundrobin; /* the process allocated by this thread */
         ++num_busy; /* the child process has been allocated */
-        busy[rr]=1; /* mark the child process as busy */
-        pthread_mutex_unlock(&mutex);
+        busy[my_process]=1; /* mark the child process as busy */
+        retval=pthread_mutex_unlock(&mutex);
+        if(retval) {
+            errno=retval;
+            ioerror("pthread_mutex_unlock");
+            longjmp(c->err, 1);
+        }
 
-        s_log(LOG_DEBUG, "Acquired libwrap process #%d", rr);
-        write_fd(ipc_socket[2*rr], c->opt->servname,
+        s_log(LOG_DEBUG, "Acquired libwrap process #%d", my_process);
+        write_fd(ipc_socket[2*my_process], c->opt->servname,
             strlen(c->opt->servname)+1, c->local_rfd.fd);
-        read_blocking(c, ipc_socket[2*rr], (u8 *)&result, sizeof(result));
-        s_log(LOG_DEBUG, "Releasing libwrap process #%d", rr);
+        read_blocking(c, ipc_socket[2*my_process],
+            (u8 *)&result, sizeof(result));
+        s_log(LOG_DEBUG, "Releasing libwrap process #%d", my_process);
 
-        pthread_mutex_lock(&mutex);
-        busy[rr]=0; /* mark the child process as free */
+        retval=pthread_mutex_lock(&mutex);
+        if(retval) {
+            errno=retval;
+            ioerror("pthread_mutex_lock");
+            longjmp(c->err, 1);
+        }
+        busy[my_process]=0; /* mark the child process as free */
         --num_busy; /* the child process has been released */
-        pthread_cond_signal(&cond); /* signal other waiting threads */
-        pthread_mutex_unlock(&mutex);
+        if(num_busy==num_processes-1) { /* need to wake up a thread */
+            retval=pthread_cond_signal(&cond); /* signal waiting threads */
+            if(retval) {
+                errno=retval;
+                ioerror("pthread_cond_signal");
+                longjmp(c->err, 1);
+            }
+        }
+        retval=pthread_mutex_unlock(&mutex);
+        if(retval) {
+            errno=retval;
+            ioerror("pthread_mutex_unlock");
+            longjmp(c->err, 1);
+        }
 
-        s_log(LOG_DEBUG, "Released libwrap process #%d", rr);
+        s_log(LOG_DEBUG, "Released libwrap process #%d", my_process);
     } else
 #endif /* USE_PTHREAD */
     { /* use original, synchronous libwrap calls */
